Package: scClassify
Type: Package
Title: scClassify: single-cell Hierarchical Classification
Version: 1.24.0
Author: Yingxin Lin
Maintainer: Yingxin Lin <yingxin.lin@sydney.edu.au>
Description: scClassify is a multiscale classification framework for single-cell RNA-seq data based on ensemble learning and cell type hierarchies, enabling sample size estimation required for accurate cell type classification and joint classification of cells using multiple references.
License: GPL-3
Encoding: UTF-8
LazyData: false
Depends: R (>= 4.0)
Imports: S4Vectors, limma, ggraph, igraph, methods, cluster,
        minpack.lm, mixtools, BiocParallel, proxy, proxyC, Matrix,
        ggplot2, hopach, diptest, mgcv, stats, graphics, statmod, Cepo
RoxygenNote: 7.1.1
Suggests: knitr, rmarkdown, BiocStyle, pkgdown
VignetteBuilder: knitr
biocViews: SingleCell, GeneExpression, Classification
BugReports: https://github.com/SydneyBioX/scClassify/issues
git_url: https://git.bioconductor.org/packages/scClassify
git_branch: RELEASE_3_23
git_last_commit: 8f9d87a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:27:01 UTC; biocbuild
