# Generated by roxygen2: do not edit by hand

export(cellTypeTrain)
export(cellTypeTree)
export(features)
export(getN)
export(learningCurve)
export(model)
export(modelweights)
export(name)
export(plotCellTypeTree)
export(predict_scClassify)
export(predict_scClassifyJoint)
export(runHOPACH)
export(runSampleCal)
export(scClassify)
export(scClassifyTrainModelList)
export(train_scClassify)
exportClasses(scClassifyTrainModel)
exportClasses(scClassifyTrainModelList)
import(ggplot2)
import(igraph)
import(limma)
import(statmod)
importClassesFrom(S4Vectors,DataFrame)
importClassesFrom(S4Vectors,SimpleList)
importClassesFrom(S4Vectors,character_OR_NULL)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(Cepo,Cepo)
importFrom(Cepo,topGenes)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(cluster,pam)
importFrom(ggraph,geom_edge_diagonal)
importFrom(ggraph,geom_node_point)
importFrom(ggraph,geom_node_text)
importFrom(ggraph,ggraph)
importFrom(graphics,abline)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(hopach,as.hdist)
importFrom(hopach,distancevector)
importFrom(hopach,hdist)
importFrom(hopach,improveordering)
importFrom(hopach,is.hdist)
importFrom(hopach,vectmatrix)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(mgcv,gam)
importFrom(minpack.lm,nlsLM)
importFrom(mixtools,normalmixEM)
importFrom(proxy,as.dist)
importFrom(proxy,dist)
importFrom(proxyC,simil)
importFrom(stats,coef)
importFrom(stats,cutree)
importFrom(stats,dnorm)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,uniroot)
